@@ -28,6 +28,7 @@
 
 import errno
 import getopt
+from glob import glob
 from packager import httpget
 import os
 import re
@@ -44,18 +45,12 @@ import urlparse
 # web server will look for repositories.
 REPOPATH="/var/www/repo"
 
-# The 10gen names for the architectures we support.
+# The MongoDB names for the architectures we support.
 ARCHES=["x86_64"]
 
 # Made up names for the flavors of distribution we package for.
-DISTROS=["debian-sysvinit", "ubuntu-upstart", "redhat"]
+DISTROS=["suse","debian","redhat","ubuntu"]
 
-# When we're preparing a directory containing packaging tool inputs
-# and our binaries, use this relative subdirectory for placing the
-# binaries.
-BINARYDIR="BINARIES"
-
-sys.stderr.write("BINARYDIR: %s, REPOPATH: %s\n" % (BINARYDIR, REPOPATH))
 
 class Spec(object):
     def __init__(self, specstr):
@@ -88,7 +83,7 @@ class Spec(object):
         elif "suffix" in self.params:
             return self.params["suffix"]
         else:
-            return "-10gen-enterprise" if int(self.ver.split(".")[1])%2==0 else "-10gen-unstable"
+            return "-enterprise" if int(self.ver.split(".")[1])%2==0 else "-enterprise-unstable"
 
 
     def pversion(self, distro):
@@ -97,8 +92,8 @@ class Spec(object):
         # will be the package's version number (but we need to know
         # our upstream version too).
         if re.search("^(debian|ubuntu)", distro.name()):
-            return re.sub("-", "~", self.ver)                
-        elif re.search("(redhat|fedora|centos)", distro.name()):
+            return re.sub("-", "~", self.ver)
+        elif re.search("(suse|redhat|fedora|centos)", distro.name()):
             return re.sub("\\d+-", "", self.ver)
         else:
             raise Exception("BUG: unsupported platform?")
@@ -108,6 +103,12 @@ class Spec(object):
             return self.params[param]
         return None
 
+    def branch(self):
+        """Return the major and minor portions of the specificed version.
+        For example, if the version is "2.5.5" the branch would be "2.5"
+        """
+        return ".".join(self.ver.split(".")[0:2])
+
 class Distro(object):
     def __init__(self, string):
         self.n=string
@@ -116,48 +117,111 @@ class Distro(object):
         return self.n
 
     def pkgbase(self):
-        # pkgbase is the first part of the package's name on
-        # this distro.
-        return "mongo" if re.search("(redhat|fedora|centos)", self.n) else "mongodb"
+        return "mongodb"
 
     def archname(self, arch):
         if re.search("^(debian|ubuntu)", self.n):
             return "i386" if arch.endswith("86") else "amd64"
-        elif re.search("^(centos|redhat|fedora)", self.n):
+        elif re.search("^(suse|centos|redhat|fedora)", self.n):
             return "i686" if arch.endswith("86") else "x86_64"
         else:
             raise Exception("BUG: unsupported platform?")
 
-    def repodir(self, arch):
+    def repodir(self, arch, build_os, spec):
         """Return the directory where we'll place the package files for
         (distro, distro_version) in that distro's preferred repository
         layout (as distinct from where that distro's packaging building
-        tools place the package files)."""
+        tools place the package files).
+
+        Examples:
+
+        repo/apt/ubuntu/dists/precise/mongodb-enterprise/2.5/multiverse/binary-amd64
+        repo/apt/ubuntu/dists/precise/mongodb-enterprise/2.5/multiverse/binary-i386
+
+        repo/apt/ubuntu/dists/trusty/mongodb-enterprise/2.5/multiverse/binary-amd64
+        repo/apt/ubuntu/dists/trusty/mongodb-enterprise/2.5/multiverse/binary-i386
+
+        repo/apt/debian/dists/wheezy/mongodb-enterprise/2.5/main/binary-amd64
+        repo/apt/debian/dists/wheezy/mongodb-enterprise/2.5/main/binary-i386
+
+        repo/yum/redhat/6/mongodb-enterprise/2.5/x86_64
+        yum/redhat/6/mongodb-enterprise/2.5/i386
+
+        repo/zypper/suse/11/mongodb-enterprise/2.5/x86_64
+        zypper/suse/11/mongodb-enterprise/2.5/i386
+
+        """
+
         if re.search("^(debian|ubuntu)", self.n):
-            return "repo/%s/dists/dist/10gen/binary-%s/" % (self.n, self.archname(arch))
+            return "repo/apt/%s/dists/%s/mongodb-enterprise/%s/%s/binary-%s/" % (self.n, self.repo_os_version(build_os), spec.branch(), self.repo_component(), self.archname(arch))
         elif re.search("(redhat|fedora|centos)", self.n):
-            return "repo/%s/os/%s/RPMS/" % (self.n, self.archname(arch))
+            return "repo/yum/%s/%s/mongodb-enterprise/%s/%s/RPMS/" % (self.n, self.repo_os_version(build_os), spec.branch(), self.archname(arch))
+        elif re.search("(suse)", self.n):
+            return "repo/zypper/%s/%s/mongodb-enterprise/%s/%s/RPMS/" % (self.n, self.repo_os_version(build_os), spec.branch(), self.archname(arch))
         else:
             raise Exception("BUG: unsupported platform?")
-    
-    def make_pkg(self, arch, spec, srcdir):
+
+    def repo_component(self):
+        """Return the name of the section/component/pool we are publishing into -
+        e.g. "multiverse" for Ubuntu, "main" for debian."""
+        if self.n == 'ubuntu':
+          return "multiverse"
+        elif self.n == 'debian':
+          return "main"
+        else:
+            raise Exception("unsupported distro: %s" % self.n)
+
+    def repo_os_version(self, build_os):
+        """Return an OS version suitable for package repo directory
+        naming - e.g. 5, 6 or 7 for redhat/centos, "precise," "wheezy," etc.
+        for Ubuntu/Debian, 11 for suse"""
+        if self.n == 'suse':
+            return re.sub(r'^suse(\d+)$', r'\1', build_os)
+        if self.n == 'redhat':
+            return re.sub(r'^rhel(\d).*$', r'\1', build_os)
+        elif self.n == 'ubuntu':
+            if build_os == 'ubuntu1204':
+                return "precise"
+            elif build_os == 'ubuntu1404':
+                return "trusty"
+            else:
+                raise Exception("unsupported build_os: %s" % build_os)
+        elif self.n == 'debian':
+            if build_os == 'debian71':
+                return 'wheezy'
+            else:
+                raise Exception("unsupported build_os: %s" % build_os)
+        else:
+            raise Exception("unsupported distro: %s" % self.n)
+
+    def make_pkg(self, build_os, arch, spec, srcdir):
         if re.search("^(debian|ubuntu)", self.n):
-            return make_deb(self, arch, spec, srcdir)
-        elif re.search("^(centos|redhat|fedora)", self.n):
-            return make_rpm(self, arch, spec, srcdir)
+            return make_deb(self, build_os, arch, spec, srcdir)
+        elif re.search("^(suse|centos|redhat|fedora)", self.n):
+            return make_rpm(self, build_os, arch, spec, srcdir)
         else:
             raise Exception("BUG: unsupported platform?")
 
     def build_os(self):
-        """Return the build os label in the binary package to download ("rhel62" 
-        for redhat, "ubuntu1204" for Ubuntu and Debian)"""
+        """Return the build os label in the binary package to download ("rhel57", "rhel62" and "rhel70"
+        for redhat, "ubuntu1204" and "ubuntu1404" for Ubuntu, "debian71" for Debian, and "suse11" for SUSE)"""
 
-        if re.search("^(debian|ubuntu)", self.n):
-            return "ubuntu1204"
-        elif re.search("(redhat|fedora|centos)", self.n):
-            return "rhel62"
+        if re.search("(suse)", self.n):
+            return [ "suse11" ]
+        if re.search("(redhat|fedora|centos)", self.n):
+            return [ "rhel70", "rhel62", "rhel57" ]
+        elif self.n == 'ubuntu':
+            return [ "ubuntu1204", "ubuntu1404" ]
+        elif self.n == 'debian':
+            return [ "debian71" ]
         else:
             raise Exception("BUG: unsupported platform?")
+
+    def release_dist(self, build_os):
+        """Return the release distribution to use in the rpm - "el5" for rhel 5.x,
+        "el6" for rhel 6.x, return anything else unchanged"""
+
+        return re.sub(r'^rh(el\d).*$', r'\1', build_os)
 def main(argv):
     (flags, specs) = parse_args(argv[1:])
     distros=[Distro(distro) for distro in DISTROS]
@@ -177,21 +241,19 @@ def main(argv):
     os.chdir(prefix)
     try:
         # Download the binaries.
-        urlfmt="http://downloads.10gen.com/linux/mongodb-linux-%s-subscription-%s-%s.tgz"
-    
+        urlfmt="http://downloads.mongodb.com/linux/mongodb-linux-%s-enterprise-%s-%s.tgz"
+
         # Build a pacakge for each distro/spec/arch tuple, and
         # accumulate the repository-layout directories.
         for (distro, spec, arch) in crossproduct(distros, specs, ARCHES):
 
-          httpget(urlfmt % (arch, distro.build_os(), spec.version()), ensure_dir(tarfile(distro, arch, spec)))
+          for build_os in distro.build_os():
+
+            httpget(urlfmt % (arch, build_os, spec.version()), ensure_dir(tarfile(build_os, arch, spec)))
+
+            repo = make_package(distro, build_os, arch, spec, srcdir)
+            make_repo(repo, distro, build_os, spec)
 
-          repos.append(make_package(distro, arch, spec, srcdir))
-    
-        # Build the repos' metadatas.
-        for repo in set(repos):
-            print repo
-            make_repo(repo)
-    
     finally:
         os.chdir(oldcwd)
     if "-n" not in flags:
@@ -206,7 +268,7 @@ def parse_args(args):
 
 Options:
 
-  -n:  Just build the packages, don't publish them as a repo 
+  -n:  Just build the packages, don't publish them as a repo
        or clean out the working directory
 
 Each SPEC is a mongodb version string optionally followed by a colon
@@ -270,26 +332,27 @@ def ensure_dir(filename):
         exc=sys.exc_value
         if exc.errno == errno.EEXIST:
             pass
-        else: 
+        else:
             raise exc
     return filename
 
 
-def tarfile(distro, arch, spec):
+def tarfile(build_os, arch, spec):
     """Return the location where we store the downloaded tarball for
-    (arch, spec)"""
-    return "dl/mongodb-linux-%s-subscription-%s-%s.tar.gz" % (spec.version(), distro.build_os(), arch)
+    this package"""
+    return "dl/mongodb-linux-%s-enterprise-%s-%s.tar.gz" % (spec.version(), build_os, arch)
 
-def setupdir(distro, arch, spec):
+def setupdir(distro, build_os, arch, spec):
     # The setupdir will be a directory containing all inputs to the
     # distro's packaging tools (e.g., package metadata files, init
     # scripts, etc), along with the already-built binaries).  In case
     # the following format string is unclear, an example setupdir
-    # would be dst/x86_64/debian-sysvinit/mongodb-10gen-unstable/
-    return "dst/%s/%s/%s%s-%s/" % (arch, distro.name(), distro.pkgbase(), spec.suffix(), spec.pversion(distro))
+    # would be dst/x86_64/debian-sysvinit/wheezy/mongodb-org-unstable/
+    # or dst/x86_64/redhat/rhel57/mongodb-org-unstable/
+    return "dst/%s/%s/%s/%s%s-%s/" % (arch, distro.name(), build_os, distro.pkgbase(), spec.suffix(), spec.pversion(distro))
 
-def unpack_binaries_into(distro, arch, spec, where):
-    """Unpack the tarfile for (distro, arch, spec) into directory where."""
+def unpack_binaries_into(build_os, arch, spec, where):
+    """Unpack the tarfile for (build_os, arch, spec) into directory where."""
     rootdir=os.getcwd()
     ensure_dir(where)
     # Note: POSIX tar doesn't require support for gtar's "-C" option,
@@ -298,21 +361,23 @@ def unpack_binaries_into(distro, arch, spec, where):
     # thing and chdir into where and run tar there.
     os.chdir(where)
     try:
-        sysassert(["tar", "xvzf", rootdir+"/"+tarfile(distro, arch, spec), "mongodb-linux-%s-subscription-%s-%s/bin" % (arch, distro.build_os(), spec.version())])
-        os.rename("mongodb-linux-%s-subscription-%s-%s/bin" % (arch, distro.build_os(), spec.version()), "bin")
-        os.rmdir("mongodb-linux-%s-subscription-%s-%s" % (arch, distro.build_os(), spec.version()))
+	sysassert(["tar", "xvzf", rootdir+"/"+tarfile(build_os, arch, spec)])
+    	release_dir = glob('mongodb-linux-*')[0]
+        for releasefile in "bin", "snmp", "LICENSE.txt", "README", "THIRD-PARTY-NOTICES":
+            os.rename("%s/%s" % (release_dir, releasefile), releasefile)
+        os.rmdir(release_dir)
     except Exception:
         exc=sys.exc_value
         os.chdir(rootdir)
         raise exc
     os.chdir(rootdir)
 
-def make_package(distro, arch, spec, srcdir):
+def make_package(distro, build_os, arch, spec, srcdir):
     """Construct the package for (arch, distro, spec), getting
     packaging files from srcdir and any user-specified suffix from
     suffixes"""
 
-    sdir=setupdir(distro, arch, spec)
+    sdir=setupdir(distro, build_os, arch, spec)
     ensure_dir(sdir)
     # Note that the RPM packages get their man pages from the debian
     # directory, so the debian directory is needed in all cases (and
@@ -321,45 +386,56 @@ def make_package(distro, arch, spec, srcdir):
         print "Copying packaging files from %s to %s" % ("%s/%s" % (srcdir, pkgdir), sdir)
         # FIXME: sh-dash-cee is bad. See if tarfile can do this.
         sysassert(["sh", "-c", "(cd \"%s\" && git archive r%s %s/ ) | (cd \"%s\" && tar xvf -)" % (srcdir, spec.version(), pkgdir, sdir)])
-    # Splat the binaries under sdir.  The "build" stages of the
-    # packaging infrastructure will move the binaries to wherever they
-    # need to go.  
-    unpack_binaries_into(distro, arch, spec, sdir+("%s/usr/"%BINARYDIR))
+    # Splat the binaries and snmp files under sdir.  The "build" stages of the
+    # packaging infrastructure will move the files to wherever they
+    # need to go.
+    unpack_binaries_into(build_os, arch, spec, sdir)
     # Remove the mongosniff binary due to libpcap dynamic
     # linkage.  FIXME: this removal should go away
     # eventually.
-    if os.path.exists(sdir+("%s/usr/bin/mongosniff"%BINARYDIR)):
-      os.unlink(sdir+("%s/usr/bin/mongosniff"%BINARYDIR))
-    return distro.make_pkg(arch, spec, srcdir)
+    if os.path.exists(sdir + "bin/mongosniff"):
+      os.unlink(sdir + "bin/mongosniff")
+    return distro.make_pkg(build_os, arch, spec, srcdir)
 
-def make_repo(repodir):
+def make_repo(repodir, distro, build_os, spec):
     if re.search("(debian|ubuntu)", repodir):
-        make_deb_repo(repodir)
-    elif re.search("(centos|redhat|fedora)", repodir):
+        make_deb_repo(repodir, distro, build_os, spec)
+    elif re.search("(suse|centos|redhat|fedora)", repodir):
         make_rpm_repo(repodir)
     else:
         raise Exception("BUG: unsupported platform?")
 
-def make_deb(distro, arch, spec, srcdir):
+def make_deb(distro, build_os, arch, spec, srcdir):
     # I can't remember the details anymore, but the initscript/upstart
     # job files' names must match the package name in some way; and
     # see also the --name flag to dh_installinit in the generated
     # debian/rules file.
     suffix=spec.suffix()
-    sdir=setupdir(distro, arch, spec)
-    if re.search("sysvinit", distro.name()):
-        os.link(sdir+"debian/init.d", sdir+"debian/%s%s.mongodb.init" % (distro.pkgbase(), suffix))
-        os.unlink(sdir+"debian/mongodb.upstart")
-    elif re.search("upstart", distro.name()):
-        os.link(sdir+"debian/mongodb.upstart", sdir+"debian/%s%s.upstart" % (distro.pkgbase(), suffix))
+    sdir=setupdir(distro, build_os, arch, spec)
+    if re.search("debian", distro.name()):
+        os.link(sdir+"debian/init.d", sdir+"debian/%s%s-server.mongod.init" % (distro.pkgbase(), suffix))
+        os.unlink(sdir+"debian/mongod.upstart")
+    elif re.search("ubuntu", distro.name()):
+        os.link(sdir+"debian/mongod.upstart", sdir+"debian/%s%s-server.mongod.upstart" % (distro.pkgbase(), suffix))
         os.unlink(sdir+"debian/init.d")
     else:
-        raise Exception("unknown debianoid flavor: not sysvinit or upstart?")
+        raise Exception("unknown debianoid flavor: not debian or ubuntu?")
     # Rewrite the control and rules files
-    write_debian_control_file(sdir+"debian/control", spec)
-    write_debian_rules_file(sdir+"debian/rules", spec)
     write_debian_changelog(sdir+"debian/changelog", spec, srcdir)
     distro_arch=distro.archname(arch)
+    sysassert(["cp", "-v", srcdir+"debian/%s%s.control" % (distro.pkgbase(), suffix), sdir+"debian/control"])
+    sysassert(["cp", "-v", srcdir+"debian/%s%s.rules" % (distro.pkgbase(), suffix), sdir+"debian/rules"])
+
+
+    # old non-server-package postinst will be hanging around for old versions
+    #
+    if os.path.exists(sdir+"debian/postinst"):
+      os.unlink(sdir+"debian/postinst")
+
+    # copy our postinst files
+    #
+    sysassert(["sh", "-c", "cp -v \"%sdebian/\"*.postinst \"%sdebian/\""%(srcdir, sdir)])
+
     # Do the packaging.
     oldcwd=os.getcwd()
     try:
@@ -367,18 +443,19 @@ def make_deb(distro, arch, spec, srcdir):
         sysassert(["dpkg-buildpackage", "-a"+distro_arch, "-k Richard Kreuter <richard@10gen.com>"])
     finally:
         os.chdir(oldcwd)
-    r=distro.repodir(arch)
+    r=distro.repodir(arch, build_os, spec)
     ensure_dir(r)
     # FIXME: see if shutil.copyfile or something can do this without
     # much pain.
-    sysassert(["cp", "-v", sdir+"../%s%s_%s%s_%s.deb"%(distro.pkgbase(), suffix, spec.pversion(distro), "-"+spec.param("revision") if spec.param("revision") else"", distro_arch), r])
+    #sysassert(["cp", "-v", sdir+"../%s%s_%s%s_%s.deb"%(distro.pkgbase(), suffix, spec.pversion(distro), "-"+spec.param("revision") if spec.param("revision") else"", distro_arch), r])
+    sysassert(["sh", "-c", "cp -v \"%s/../\"*.deb \"%s\""%(sdir, r)])
     return r
 
-def make_deb_repo(repo):
+def make_deb_repo(repo, distro, build_os, spec):
     # Note: the Debian repository Packages files must be generated
     # very carefully in order to be usable.
     oldpwd=os.getcwd()
-    os.chdir(repo+"../../../../")
+    os.chdir(repo+"../../../../../../")
     try:
         dirs=set([os.path.dirname(deb)[2:] for deb in backtick(["find", ".", "-name", "*.deb"]).split()])
         for d in dirs:
@@ -399,16 +476,14 @@ def make_deb_repo(repo):
     # Notes: the Release{,.gpg} files must live in a special place,
     # and must be created after all the Packages.gz files have been
     # done.
-    s="""
-Origin: 10gen
-Label: 10gen
-Suite: 10gen
-Codename: %s
-Version: %s
-Architectures: i386 amd64
-Components: 10gen
-Description: 10gen packages
-""" % ("dist", "dist")
+    s="""Origin: mongodb
+Label: mongodb
+Suite: mongodb
+Codename: %s/mongodb-enterprise
+Architectures: amd64
+Components: %s
+Description: MongoDB packages
+""" % (distro.repo_os_version(build_os), distro.repo_component())
     if os.path.exists(repo+"../../Release"):
         os.unlink(repo+"../../Release")
     if os.path.exists(repo+"../../Release.gpg"):
@@ -475,7 +550,7 @@ def move_repos_into_place(src, dst):
             exc=sys.exc_value
             if exc.errno == errno.EEXIST:
                 pass
-            else: 
+            else:
                 raise exc
         i=i+1
 
@@ -493,7 +568,7 @@ def move_repos_into_place(src, dst):
                exc=sys.exc_value
                if exc.errno == errno.EEXIST:
                    pass
-               else: 
+               else:
                    raise exc
 
     os.rename(tmpnam, dst)
@@ -531,182 +606,19 @@ def write_debian_changelog(path, spec, srcdir):
     finally:
         f.close()
 
-def write_debian_control_file(path, spec):
-    s="""Source: @@PACKAGE_BASENAME@@
-Section: devel
-Priority: optional
-Maintainer: Richard Kreuter <richard@10gen.com>
-Build-Depends: 
-Standards-Version: 3.8.0
-Homepage: http://www.mongodb.org
-
-Package: @@PACKAGE_BASENAME@@
-Conflicts: @@PACKAGE_CONFLICTS@@
-Architecture: any
-Depends: libc6 (>= 2.3.2), libgcc1 (>= 1:4.1.1), libstdc++6 (>= 4.1.1), libsnmp15, libsasl2-2, libssl1.0.0
-Description: An object/document-oriented database
- MongoDB is a high-performance, open source, schema-free 
- document-oriented  data store that's easy to deploy, manage
- and use. It's network accessible, written in C++ and offers
- the following features :
- .
-    * Collection oriented storage - easy storage of object-
-      style data
-    * Full index support, including on inner objects
-    * Query profiling
-    * Replication and fail-over support
-    * Efficient storage of binary data including large 
-      objects (e.g. videos)
-    * Auto-sharding for cloud-level scalability (Q209)
- .
- High performance, scalability, and reasonable depth of
- functionality are the goals for the project.
-"""
-    s=re.sub("@@PACKAGE_BASENAME@@", "mongodb%s" % spec.suffix(), s)
-    conflict_suffixes=["", "-stable", "-unstable", "-nightly", "-10gen", "-10gen-unstable", "-10gen-enterprise"]
-    conflict_suffixes = [suff for suff in conflict_suffixes if suff != spec.suffix()]
-    s=re.sub("@@PACKAGE_CONFLICTS@@", ", ".join(["mongodb"+suffix for suffix in conflict_suffixes] + [ "mongodb18"+suffix for suffix in conflict_suffixes] + ["mongodb20"+suffix for suffix in conflict_suffixes]), s)
-
-    f=open(path, 'w')
-    try:
-        f.write(s)
-    finally:
-        f.close()
-
-def write_debian_rules_file(path, spec):
-    # Note debian/rules is a makefile, so for visual disambiguation we
-    # make all tabs here \t.
-    s="""#!/usr/bin/make -f
-# -*- makefile -*-
-# Sample debian/rules that uses debhelper.
-# This file was originally written by Joey Hess and Craig Small.
-# As a special exception, when this file is copied by dh-make into a
-# dh-make output file, you may use that output file without restriction.
-# This special exception was added by Craig Small in version 0.37 of dh-make.
-
-# Uncomment this to turn on verbose mode.
-#export DH_VERBOSE=1
-
-
-configure: configure-stamp
-configure-stamp:
-\tdh_testdir
-        # Add here commands to configure the package.
-
-\ttouch configure-stamp
-
-
-build: build-stamp
-
-build-stamp: configure-stamp  
-\tdh_testdir
-
-        # Add here commands to compile the package.
-# THE FOLLOWING LINE IS INTENTIONALLY COMMENTED. 
-\t# scons 
-        #docbook-to-man debian/mongodb.sgml > mongodb.1
-\tls debian/*.1 > debian/@@PACKAGE_NAME@@.manpages
-
-\ttouch $@
-
-clean: 
-\tdh_testdir
-\tdh_testroot
-\trm -f build-stamp configure-stamp
-
-\t# FIXME: scons freaks out at the presence of target files
-\t# under debian/mongodb.
-\t#scons -c
-\trm -rf $(CURDIR)/debian/@@PACKAGE_NAME@@
-\trm -f config.log
-\trm -f mongo
-\trm -f mongod
-\trm -f mongoimportjson
-\trm -f mongoexport
-\trm -f mongorestore
-\trm -f mongodump
-\trm -f mongofiles
-\trm -f .sconsign.dblite
-\trm -f libmongoclient.a
-\trm -rf client/*.o
-\trm -rf tools/*.o
-\trm -rf shell/*.o
-\trm -rf .sconf_temp
-\trm -f buildscripts/*.pyc 
-\trm -f *.pyc
-\trm -f buildinfo.cpp
-\tdh_clean debian/files
-
-install: build
-\tdh_testdir
-\tdh_testroot
-\tdh_prep
-\tdh_installdirs
-
-# THE FOLLOWING LINE IS INTENTIONALLY COMMENTED.
-\t# scons --prefix=$(CURDIR)/debian/mongodb/usr install
-\tcp -v $(CURDIR)/@@BINARYDIR@@/usr/bin/* $(CURDIR)/debian/@@PACKAGE_NAME@@/usr/bin
-\tmkdir -p $(CURDIR)/debian/@@PACKAGE_NAME@@/etc
-\tcp $(CURDIR)/debian/mongodb.conf $(CURDIR)/debian/@@PACKAGE_NAME@@/etc/mongodb.conf 
-
-\tmkdir -p $(CURDIR)/debian/@@PACKAGE_NAME@@/usr/share/lintian/overrides/
-\tinstall -m 644 $(CURDIR)/debian/lintian-overrides \
-\t\t$(CURDIR)/debian/@@PACKAGE_NAME@@/usr/share/lintian/overrides/@@PACKAGE_NAME@@
-
-# Build architecture-independent files here.
-binary-indep: build install
-# We have nothing to do by default.
-
-# Build architecture-dependent files here.
-binary-arch: build install
-\tdh_testdir
-\tdh_testroot
-\tdh_installchangelogs 
-\tdh_installdocs
-\tdh_installexamples
-#\tdh_install
-#\tdh_installmenu
-#\tdh_installdebconf\t
-#\tdh_installlogrotate
-#\tdh_installemacsen
-#\tdh_installpam
-#\tdh_installmime
-\tdh_installinit --name=@@PACKAGE_BASENAME@@
-#\tdh_installinfo
-\tdh_installman
-\tdh_link
-# Appears to be broken on Ubuntu 11.10...?
-#\tdh_strip
-\tdh_compress
-\tdh_fixperms
-\tdh_installdeb
-\tdh_shlibdeps
-\tdh_gencontrol
-\tdh_md5sums
-\tdh_builddeb
-
-binary: binary-indep binary-arch
-.PHONY: build clean binary-indep binary-arch binary install configure
-"""
-    s=re.sub("@@PACKAGE_NAME@@", "mongodb%s" % spec.suffix(), s)
-    s=re.sub("@@PACKAGE_BASENAME@@", "mongodb", s)
-    s=re.sub("@@BINARYDIR@@", BINARYDIR, s)
-    f=open(path, 'w')
-    try:
-        f.write(s)
-    finally:
-        f.close()
-    # FIXME: some versions of debianoids seem to
-    # need the rules file to be 755?
-    os.chmod(path, stat.S_IXUSR|stat.S_IWUSR|stat.S_IRUSR|stat.S_IXGRP|stat.S_IRGRP|stat.S_IXOTH|stat.S_IWOTH)
-
-def make_rpm(distro, arch, spec, srcdir):
+def make_rpm(distro, build_os, arch, spec, srcdir):
     # Create the specfile.
     suffix=spec.suffix()
-    sdir=setupdir(distro, arch, spec)
-    specfile=sdir+"rpm/mongo%s.spec" % suffix
-    write_rpm_spec_file(specfile, spec)
-    topdir=ensure_dir(os.getcwd()+'/rpmbuild/')
+    sdir=setupdir(distro, build_os, arch, spec)
+
+    # Use special suse init script if we're building for SUSE 
+    #
+    if distro.name() == "suse":
+        os.unlink(sdir+"rpm/init.d-mongod")
+        os.link(sdir+"rpm/init.d-mongod.suse", sdir+"rpm/init.d-mongod")
+
+    specfile=srcdir+"rpm/mongodb%s.spec" % suffix
+    topdir=ensure_dir('%s/rpmbuild/%s/' % (os.getcwd(), build_os))
     for subdir in ["BUILD", "RPMS", "SOURCES", "SPECS", "SRPMS"]:
         ensure_dir("%s/%s/" % (topdir, subdir))
     distro_arch=distro.archname(arch)
@@ -729,7 +641,8 @@ def make_rpm(distro, arch, spec, srcdir):
     macrofiles=[l for l in backtick(["rpm", "--showrc"]).split("\n") if l.startswith("macrofiles")]
     flags=[]
     macropath=os.getcwd()+"/macros"
-    write_rpm_macros_file(macropath, topdir)
+
+    write_rpm_macros_file(macropath, topdir, distro.release_dist(build_os))
     if len(macrofiles)>0:
         macrofiles=macrofiles[0]+":"+macropath
         rcfile=os.getcwd()+"/rpmrc"
@@ -747,12 +660,12 @@ def make_rpm(distro, arch, spec, srcdir):
     oldcwd=os.getcwd()
     os.chdir(sdir+"/../")
     try:
-        sysassert(["tar", "-cpzf", topdir+"SOURCES/mongo%s-%s.tar.gz" % (suffix, spec.pversion(distro)), os.path.basename(os.path.dirname(sdir))])
+        sysassert(["tar", "-cpzf", topdir+"SOURCES/mongodb%s-%s.tar.gz" % (suffix, spec.pversion(distro)), os.path.basename(os.path.dirname(sdir))])
     finally:
         os.chdir(oldcwd)
     # Do the build.
-    sysassert(["rpmbuild", "-ba", "--target", distro_arch] + flags + ["%s/SPECS/mongo%s.spec" % (topdir, suffix)])
-    r=distro.repodir(arch)
+    sysassert(["rpmbuild", "-ba", "--target", distro_arch] + flags + ["%s/SPECS/mongodb%s.spec" % (topdir, suffix)])
+    r=distro.repodir(arch, build_os, spec)
     ensure_dir(r)
     # FIXME: see if some combination of shutil.copy<hoohah> and glob
     # can do this without shelling out.
@@ -775,221 +688,12 @@ def write_rpmrc_file(path, string):
     finally:
         f.close()
 
-def write_rpm_macros_file(path, topdir):
+def write_rpm_macros_file(path, topdir, release_dist):
     f=open(path, 'w')
     try:
-        f.write("%%_topdir	%s" % topdir)
-    finally:
-        f.close()
-
-def write_rpm_spec_file(path, spec):
-    s="""Name: @@PACKAGE_BASENAME@@
-Conflicts: @@PACKAGE_CONFLICTS@@
-Obsoletes: @@PACKAGE_OBSOLETES@@
-Version: @@PACKAGE_VERSION@@
-Release: mongodb_@@PACKAGE_REVISION@@%{?dist}
-Summary: mongo client shell and tools
-License: AGPL 3.0
-URL: http://www.mongodb.org
-Group: Applications/Databases
-Requires: cyrus-sasl, net-snmp-libs
-
-Source0: %{name}-%{version}.tar.gz
-BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
-
-%description
-Mongo (from "huMONGOus") is a schema-free document-oriented database.
-It features dynamic profileable queries, full indexing, replication
-and fail-over support, efficient storage of large binary data objects,
-and auto-sharding.
-
-This package provides the mongo shell, import/export tools, and other
-client utilities.
-
-%package server
-Summary: mongo server, sharding server, and support scripts
-Group: Applications/Databases
-Requires: @@PACKAGE_BASENAME@@
-
-%description server
-Mongo (from "huMONGOus") is a schema-free document-oriented database.
-
-This package provides the mongo server software, mongo sharding server
-softwware, default configuration files, and init.d scripts.
-
-%package devel
-Summary: Headers and libraries for mongo development. 
-Group: Applications/Databases
-
-%description devel
-Mongo (from "huMONGOus") is a schema-free document-oriented database.
-
-This package provides the mongo static library and header files needed
-to develop mongo client software.
-
-%prep
-%setup
-
-%build
-#scons --prefix=$RPM_BUILD_ROOT/usr all
-# XXX really should have shared library here
-
-%install
-#scons --prefix=$RPM_BUILD_ROOT/usr install
-mkdir -p $RPM_BUILD_ROOT/usr
-cp -rv @@BINARYDIR@@/usr/bin $RPM_BUILD_ROOT/usr
-mkdir -p $RPM_BUILD_ROOT/usr/share/man/man1
-cp debian/*.1 $RPM_BUILD_ROOT/usr/share/man/man1/
-# FIXME: remove this rm when mongosniff is back in the package
-rm -v $RPM_BUILD_ROOT/usr/share/man/man1/mongosniff.1*
-mkdir -p $RPM_BUILD_ROOT/etc/rc.d/init.d
-cp -v rpm/init.d-mongod $RPM_BUILD_ROOT/etc/rc.d/init.d/mongod
-chmod a+x $RPM_BUILD_ROOT/etc/rc.d/init.d/mongod
-mkdir -p $RPM_BUILD_ROOT/etc
-cp -v rpm/mongod.conf $RPM_BUILD_ROOT/etc/mongod.conf
-mkdir -p $RPM_BUILD_ROOT/etc/sysconfig
-cp -v rpm/mongod.sysconfig $RPM_BUILD_ROOT/etc/sysconfig/mongod
-mkdir -p $RPM_BUILD_ROOT/var/lib/mongo
-mkdir -p $RPM_BUILD_ROOT/var/log/mongo
-touch $RPM_BUILD_ROOT/var/log/mongo/mongod.log
-
-%clean
-#scons -c
-rm -rf $RPM_BUILD_ROOT
-
-%pre server
-if ! /usr/bin/id -g mongod &>/dev/null; then
-    /usr/sbin/groupadd -r mongod
-fi
-if ! /usr/bin/id mongod &>/dev/null; then
-    /usr/sbin/useradd -M -r -g mongod -d /var/lib/mongo -s /bin/false \
-	-c mongod mongod > /dev/null 2>&1
-fi
-
-%post server
-if test $1 = 1
-then
-  /sbin/chkconfig --add mongod
-fi
-
-%preun server
-if test $1 = 0
-then
-  /sbin/chkconfig --del mongod
-fi
-
-%postun server
-if test $1 -ge 1
-then
-  /sbin/service mongod condrestart >/dev/null 2>&1 || :
-fi
-
-%files
-%defattr(-,root,root,-)
-#%doc README GNU-AGPL-3.0.txt
-
-%{_bindir}/bsondump
-%{_bindir}/mongo
-%{_bindir}/mongodump
-%{_bindir}/mongoexport
-#@@VERSION!=2.1.0@@%{_bindir}/mongofiles
-%{_bindir}/mongoimport
-#@@VERSION>=2.1.0@@%{_bindir}/mongooplog
-#@@VERSION>=2.1.0@@%{_bindir}/mongoperf
-%{_bindir}/mongorestore
-#@@VERSION>1.9@@%{_bindir}/mongotop
-%{_bindir}/mongostat
-# FIXME: uncomment when mongosniff is back in the package
-#%{_bindir}/mongosniff
-
-# FIXME: uncomment this when there's a stable release whose source
-# tree contains a bsondump man page.
-#@@VERSION>1.9@@%{_mandir}/man1/bsondump.1*
-%{_mandir}/man1/mongo.1*
-%{_mandir}/man1/mongodump.1*
-%{_mandir}/man1/mongoexport.1*
-%{_mandir}/man1/mongofiles.1*
-%{_mandir}/man1/mongoimport.1*
-%{_mandir}/man1/mongorestore.1*
-%{_mandir}/man1/mongostat.1*
-# FIXME: uncomment when mongosniff is back in the package
-#%{_mandir}/man1/mongosniff.1*
-#@@VERSION>=2.4.0@@%{_mandir}/man1/mongotop.1*
-#@@VERSION>=2.4.0@@%{_mandir}/man1/mongoperf.1*
-#@@VERSION>=2.4.0@@%{_mandir}/man1/mongooplog.1*
-
-%files server
-%defattr(-,root,root,-)
-%config(noreplace) /etc/mongod.conf
-%{_bindir}/mongod
-%{_bindir}/mongos
-%{_mandir}/man1/mongod.1*
-%{_mandir}/man1/mongos.1*
-/etc/rc.d/init.d/mongod
-/etc/sysconfig/mongod
-#/etc/rc.d/init.d/mongos
-%attr(0755,mongod,mongod) %dir /var/lib/mongo
-%attr(0755,mongod,mongod) %dir /var/log/mongo
-%attr(0640,mongod,mongod) %config(noreplace) %verify(not md5 size mtime) /var/log/mongo/mongod.log
-
-%changelog
-* Thu Jan 28 2010 Richard M Kreuter <richard@10gen.com>
-- Minor fixes.
-
-* Sat Oct 24 2009 Joe Miklojcik <jmiklojcik@shopwiki.com> - 
-- Wrote mongo.spec.
-"""
-    suffix=spec.suffix()
-    s=re.sub("@@PACKAGE_BASENAME@@", "mongo%s" % suffix, s)
-    s=re.sub("@@PACKAGE_VERSION@@", spec.pversion(Distro("redhat")), s)
-    # FIXME, maybe: the RPM guide says that Release numbers ought to
-    # be integers starting at 1, but we use "mongodb_1{%dist}",
-    # whatever the hell that means.
-    s=re.sub("@@PACKAGE_REVISION@@", str(int(spec.param("revision"))+1) if spec.param("revision") else "1", s)
-    s=re.sub("@@BINARYDIR@@", BINARYDIR, s)
-    conflict_suffixes=["", "-10gen", "-10gen-unstable", "-10gen-enterprise"]
-    conflict_suffixes = [suff for suff in conflict_suffixes if suff != spec.suffix()]
-    s=re.sub("@@PACKAGE_CONFLICTS@@", ", ".join(["mongo"+suffix for suffix in conflict_suffixes] + [ "mongo18"+suffix for suffix in conflict_suffixes] + ["mongo20"+suffix for suffix in conflict_suffixes]), s)
-    if suffix.endswith("-10gen"):
-        s=re.sub("@@PACKAGE_PROVIDES@@", "mongo-stable", s)
-        s=re.sub("@@PACKAGE_OBSOLETES@@", "mongo-stable", s)
-    elif suffix == "-10gen-unstable":
-        s=re.sub("@@PACKAGE_PROVIDES@@", "mongo-unstable", s)
-        s=re.sub("@@PACKAGE_OBSOLETES@@", "mongo-unstable", s)
-    elif suffix == "-10gen-enterprise":
-        s=re.sub("@@PACKAGE_PROVIDES@@", "mongo-enterprise", s)
-        s=re.sub("@@PACKAGE_OBSOLETES@@", "mongo-enterprise", s)
-    else:
-        raise Exception("BUG: unknown suffix %s" % suffix)
-
-    lines=[]
-    for line in s.split("\n"):
-        m = re.search("@@VERSION(>|>=|!=)(\d.*)@@(.*)", line)
-        if m:
-          op = m.group(1)
-          ver = m.group(2)
-          fn = m.group(3)
-          if op == '>':
-            if spec.version_better_than(ver):
-              lines.append(fn)
-          elif op == '>=':
-            if spec.version() == ver or spec.version_better_than(ver):
-              lines.append(fn)
-          elif op == '!=':
-            if spec.version() != ver:
-              lines.append(fn)
-          else:
-            # Since we're inventing our own template system for RPM
-            # specfiles here, we oughtn't use template syntax we don't
-            # support.
-            raise Exception("BUG: probable bug in packager script: %s, %s, %s" % (m.group(1), m.group(2), m.group(3)))
-        else:
-            lines.append(line)
-    s="\n".join(lines)
-
-    f=open(path, 'w')
-    try:
-        f.write(s)
+        f.write("%%_topdir	%s\n" % topdir)
+        f.write("%%dist	.%s\n" % release_dist)
+        f.write("%_use_internal_dependency_generator 0\n")
     finally:
         f.close()
 
