@@ -13,7 +13,7 @@ setupMoveChunkTest(st);
 var shards = [st.shard0, st.shard1];
 var foundMoveChunk = false;
 for(i in shards) {
-    var dbpath = shards[i].adminCommand("getCmdLineOpts").parsed.dbpath;
+    var dbpath = shards[i].adminCommand("getCmdLineOpts").parsed.storage.dbPath;
     var hasMoveChunkDir = 0 != ls(dbpath).filter(function(a) {return null != a.match("moveChunk")}).length
     foundMoveChunk = foundMoveChunk || hasMoveChunkDir;
 }
