@@ -29,17 +29,18 @@
 #pragma once
 
 #include "mongo/pch.h"
-#include "../bson/util/builder.h"
-#include "../util/optime.h"
-#include "../bson/bsontypes.h"
-#include "../bson/oid.h"
-#include "../bson/bsonelement.h"
-#include "../bson/bsonobj.h"
-#include "../bson/bsonmisc.h"
-#include "../bson/bsonobjbuilder.h"
-#include "../bson/bsonobjiterator.h"
-#include "../bson/bson-inl.h"
-#include "../bson/ordering.h"
+
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/optime.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsonobjiterator.h"
+#include "mongo/bson/bson-inl.h"
+#include "mongo/bson/ordering.h"
 #include "mongo/base/string_data.h"
-#include "../bson/bson_db.h"
+#include "mongo/bson/bson_db.h"
 
