@@ -12,12 +12,25 @@
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*
+*    As a special exception, the copyright holders give permission to link the
+*    code of portions of this program with the OpenSSL library under certain
+*    conditions as described in each individual source file and distribute
+*    linked combinations including the program with the OpenSSL library. You
+*    must comply with the GNU Affero General Public License in all respects for
+*    all of the code used other than as permitted herein. If you modify file(s)
+*    with this exception, you may extend this exception to your version of the
+*    file(s), but you are not obligated to do so. If you do not wish to do so,
+*    delete this exception statement from your version. If you delete this
+*    exception statement from all source files in the program, then also delete
+*    it in the license file.
 */
 
-#include "pch.h"
-#include "../commands.h"
-#include "rs.h"
-#include "multicmd.h"
+#include "mongo/pch.h"
+
+#include "mongo/db/commands.h"
+#include "mongo/db/repl/multicmd.h"
+#include "mongo/db/repl/rs.h"
 
 namespace mongo {
 
@@ -26,13 +39,14 @@ namespace mongo {
         */
     class CmdReplSetFresh : public ReplSetCommand {
     public:
+        void help(stringstream& h) const { h << "internal"; }
         CmdReplSetFresh() : ReplSetCommand("replSetFresh") { }
         virtual void addRequiredPrivileges(const std::string& dbname,
                                            const BSONObj& cmdObj,
                                            std::vector<Privilege>* out) {
             ActionSet actions;
-            actions.addAction(ActionType::replSetFresh);
-            out->push_back(Privilege(AuthorizationManager::SERVER_RESOURCE_NAME, actions));
+            actions.addAction(ActionType::internal);
+            out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
         }
     private:
 
@@ -43,6 +57,12 @@ namespace mongo {
                 return false;
             }
 
+            int cfgver = cmdObj["cfgver"].Int();
+            if (theReplSet->config().version < cfgver) {
+                // Our config is stale; do not veto.
+                return false;
+            }
+
             unsigned id = cmdObj["id"].Int();
             const Member* primary = theReplSet->box.getPrimary();
             const Member* hopeful = theReplSet->findById(id);
@@ -125,13 +145,14 @@ namespace mongo {
 
     class CmdReplSetElect : public ReplSetCommand {
     public:
+        void help(stringstream& h) const { h << "internal"; }
         CmdReplSetElect() : ReplSetCommand("replSetElect") { }
         virtual void addRequiredPrivileges(const std::string& dbname,
                                            const BSONObj& cmdObj,
                                            std::vector<Privilege>* out) {
             ActionSet actions;
-            actions.addAction(ActionType::replSetElect);
-            out->push_back(Privilege(AuthorizationManager::SERVER_RESOURCE_NAME, actions));
+            actions.addAction(ActionType::internal);
+            out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
         }
     private:
         virtual bool run(const string& , BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
@@ -370,7 +391,7 @@ namespace mongo {
 
         rs.sethbmsg("",9);
 
-        if( !allUp && time(0) - started < 60 * 5 ) {
+        if (!allUp && time(0) - serverGlobalParams.started < 60 * 5) {
             /* the idea here is that if a bunch of nodes bounce all at once, we don't want to drop data
                if we don't have to -- we'd rather be offline and wait a little longer instead
                todo: make this configurable.
@@ -442,6 +463,10 @@ namespace mongo {
                     /* succeeded. */
                     LOG(1) << "replSet election succeeded, assuming primary role" << rsLog;
                     success = true;
+                    {
+                        mutex::scoped_lock lk(OpTime::m);
+                        setElectionTime(OpTime::now(lk));
+                    }
                     rs.assumePrimary();
                 }
             }
