@@ -21,16 +21,16 @@
 
 #include <iostream>
 
-#include "mongo/platform/basic.h"
+#include "mongo/util/log.h"
 
 namespace mongo {
 
-    // Print stack trace information to "os", default to std::cout.
-    void printStackTrace(std::ostream &os=std::cout);
+    // Print stack trace information to "os", default to the log stream.
+    void printStackTrace(std::ostream &os=log().stream());
 
 #if defined(_WIN32)
-    // Print stack trace (using a specified stack context) to "os", default to std::cout.
-    void printWindowsStackTrace(CONTEXT &context, std::ostream &os=std::cout);
+    // Print stack trace (using a specified stack context) to "os", default to the log stream.
+    void printWindowsStackTrace(CONTEXT &context, std::ostream &os=log().stream());
 
     // Print error message from C runtime followed by stack trace
     int crtDebugCallback(int, char* originalMessage, int*);
